//
// Cleversafe open-source code header - Version 1.2 - February 15, 2008
//
// Cleversafe Dispersed Storage(TM) is software for secure, private and
// reliable storage of the world's data using information dispersal.
//
// Copyright (C) 2005-2008 Cleversafe, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
// USA.
//
// Contact Information: Cleversafe, 224 North Desplaines Street, Suite 500 
// Chicago IL 60661
// email licensing@cleversafe.org
//
// END-OF-HEADER
//-----------------------
// Author: mmotwani
//
// Date: Jun 1, 2007
//---------------------

package org.cleversafe.layer.protocol;

/**
 * Interface that must be implemented by all protocol messages that contain a sequence number. The
 * sequence number is set and used by the network layer to retain the ordering of all the protocol
 * messages that are sent or received.
 * 
 * A response received generally contains the same sequence number as its corresponding request that
 * was sent.
 * 
 * @see ProtocolMessage
 */
public interface SequencedProtocolMessage extends ProtocolMessage
{
   /**
    * Returns the sequence number of this protocol message.
    * 
    * @return
    */
   int getSequenceNumber();

   /**
    * Sets the sequence number of this protocol message.
    * 
    * @param sequenceNumber
    */
   void setSequenceNumber(int sequenceNumber);
}
