//
// Cleversafe open-source code header - Version 1.2 - February 15, 2008
//
// Cleversafe Dispersed Storage(TM) is software for secure, private and
// reliable storage of the world's data using information dispersal.
//
// Copyright (C) 2005-2008 Cleversafe, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
// USA.
//
// Contact Information: Cleversafe, 224 North Desplaines Street, Suite 500 
// Chicago IL 60661
// email licensing@cleversafe.org
//
// END-OF-HEADER
//-----------------------
// @author: mmotwani
//
// Date: Oct 8, 2007
//---------------------

package org.cleversafe.serialization.raw.custom;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import org.cleversafe.layer.protocol.MultipleRemoveResponse;
import org.cleversafe.serialization.GridSerializable;
import org.cleversafe.serialization.exceptions.EncodeDecodeException;

public class MultipleRemoveResponseSerialization extends BaseResponseSerialization
{
   public MultipleRemoveResponseSerialization()
   {
      super(MultipleRemoveResponse.class);
   }

   @Override
   public GridSerializable read(DataInput in) throws EncodeDecodeException, IOException
   {
      MultipleRemoveResponse response = new MultipleRemoveResponse();

      // Read the exception
      readException(in, response);

      // Read the removed booleans
      boolean[] removed = new boolean[in.readInt()];
      for (int i = 0; i < removed.length; ++i)
      {
         removed[i] = in.readBoolean();
      }
      response.setRemoved(removed);

      return response;
   }

   @Override
   public void write(DataOutput out, GridSerializable serializable) throws EncodeDecodeException,
         IOException
   {
      assert serializable instanceof MultipleRemoveResponse;
      MultipleRemoveResponse response = (MultipleRemoveResponse) serializable;

      if (response.getRemoved() == null)
      {
         throw new EncodeDecodeException(
               "Removed boolean array in MultipleRemoveResponse must not be null");
      }

      // Write the exception
      writeException(out, response);

      // Write the removed booleans
      out.writeInt(response.getRemoved().length);
      for (int i = 0; i < response.getRemoved().length; ++i)
      {
         out.writeBoolean(response.getRemoved()[i]);
      }
   }

   @Override
   public int getMaxSerializedSize(GridSerializable serializable)
   {
      assert serializable instanceof MultipleRemoveResponse;
      MultipleRemoveResponse response = (MultipleRemoveResponse) serializable;

      int size = super.getMaxSerializedSize(serializable);
      size += 4; // array size
      if (response.getRemoved() != null)
      {
         size += response.getRemoved().length; // 1 byte for each boolean
      }
      return size;
   }
}
