//Cleversafe open-source code header - Version 1.1 - December 1, 2006
//
//Cleversafe Dispersed Storage(TM) is software for secure, private and
//reliable storage of the world's data using information dispersal.
//
//Copyright (C) 2005-2007 Cleversafe, Inc.
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
//USA.
//
//Contact Information: 
// Cleversafe, 10 W. 35th Street, 16th Floor #84,
// Chicago IL 60616
// email: licensing@cleversafe.org
//
//END-OF-HEADER
//-----------------------
//@author: John Quigley <jquigley@cleversafe.com>
//@date: January 1, 2008
//---------------------

package org.jscsi.scsi.transport;

import java.nio.ByteBuffer;

import org.jscsi.scsi.tasks.Status;

/**
 * Asynchronous handler used to handle confirmation of a complete command.
 */
public interface ConfirmationHandler
{
   /**
    * Handles confirmation of a complete command.
    * 
    * @param response The service response from the transport layer.
    * @param nexus The originating command Nexus.
    * @param input The input buffer. This is the same object as passed into
    *    {@link InitiatorTransportPort#send(Command, ByteBuffer, ByteBuffer, ConfirmationHandler)}.
    *    Incoming input data is written to the input buffer before the handler is called.
    *    <code>null</code> only if the command did not require an input buffer and the send
    *    caller did not provide one. 
    * @param status The returned command status.
    * @param senseData Any autosense data.
    */
   void handle(
         ServiceResponse response,
         Nexus nexus,
         ByteBuffer input,
         Status status,
         ByteBuffer senseData);
}
